@@ -488,8 +488,6 @@ int Client::exec()
 
     while (mState != STATE_EXIT)
     {
-        bool handledEvents = false;
-
         if (game)
         {
             // Let the game handle the events while it is active
@@ -500,8 +498,6 @@ int Client::exec()
             // Handle SDL events
             while (SDL_PollEvent(&event))
             {
-                handledEvents = true;
-
                 switch (event.type)
                 {
                     case SDL_QUIT:
